@@ -260,6 +260,7 @@ int	*lport;
 			socks_nsin.sin_addr.s_addr = inet_addr(socks_server);
 		else
 			bcopy(hp->h_addr_list[0], &socks_nsin.sin_addr, hp->h_length);
+/*
 		if (connect(sock, (struct sockaddr *)&socks_nsin, sizeof(struct sockaddr_in)) == 0)
 			return 0;
 		else {
@@ -274,6 +275,7 @@ int	*lport;
 			if (!(socks_serverlist)) {
 				return -1;
 			}
+*/
 #if defined(SUPPORT_RCMD)
 			if ((*lport < IPPORT_RESERVED) && (*lport >= IPPORT_RESERVED/2))
 				new_sock = rresvport(lport);
@@ -285,12 +287,27 @@ int	*lport;
 			if (new_sock < 0) {
 				return -1;
 			}
+		if (connect(new_sock, (struct sockaddr *)&socks_nsin, sizeof(struct sockaddr_in)) == 0) {
 			if (dup2(new_sock, sock) < 0) {
 				close(new_sock);
 				return -1;
 			} else {
 				close(new_sock);
 			}
+			return 0;
+		} else {
+			close(new_sock);
+#ifdef SVR4
+			if ((errno == EISCONN) || (errno == EINPROGRESS) || (errno == EAGAIN))
+#else
+			if ((errno == EISCONN) || (errno == EINPROGRESS))
+#endif
+				return -1;
+			syslog(LOG_LOW, "Failed to connect to sockd at %s: %m",
+				socks_server);
+			if (!(socks_serverlist)) {
+				return -1;
+			}
 		}
 /* >>> YDL 94/11/14 */
 		if (socks_serverlist) *socks_serverlist++ =',';
@@ -359,7 +376,8 @@ Socks_t	*dst;
 		if (sta == -1)
 			perror("select in socks_GetDst");
 		else 
-			perror("read in socks_GetDst");
+			fprintf(stderr, "SOCKS server closed connection\n");
+		
 		return(sta);
 	}
 	return(0);
@@ -448,7 +466,11 @@ fprintf(stderr, "Rconnect 100, Rconnect(%d, %s, %s)\n", sock, socks_dst_name, so
 #ifdef DEBUG
 fprintf(stderr, "Rconnect 200, Rconnect(%d, %s, %s)\n", sock, socks_dst_name, socks_dst_serv);
 #endif /* #ifdef DEBUG */
+#if defined(NO_WAITPID)
+		wait_ret = wait3(&status, WNOHANG, (struct rusage *) NULL);
+#else
 		wait_ret = waitpid(socks_conn_init, &status, WNOHANG);
+#endif /* #if defined(NO_WAITPID) */
 		if (wait_ret == 0) {
 			errno = EALREADY;
 #ifdef DEBUG
@@ -851,7 +873,11 @@ struct timeval *timeout;
 	if ((writefds == NULL) || !FD_ISSET(socks_conn_sock, writefds))
 		return(select(width, readfds, writefds, exceptfds, timeout));
 
+#if defined(NO_WAITPID)
+	wait_ret = wait3(&status, WNOHANG, (struct rusage *) NULL);
+#else
 	wait_ret = waitpid(socks_conn_init, &status, WNOHANG);
+#endif /* #if defined(NO_WAITPID) */
 	if (wait_ret == 0) {
 		FD_CLR(socks_conn_sock, writefds);
 		return(select(width, readfds, writefds, exceptfds, timeout));
