@@ -1,5 +1,6 @@
 #include <pwd.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <sys/signal.h>
@@ -21,7 +22,7 @@
 #if defined(AIX)
 #include <sys/mode.h>
 #endif
-#if defined(sgi) && defined(SYSTYPE_SVR4)
+#if (defined(sgi) && defined(SYSTYPE_SVR4)) || defined(__FreeBSD__)
 #include <sys/stat.h>
 #endif
 
@@ -567,14 +568,14 @@ Socks_t	*dst;
 	if (listen(out, 1) < 0)
 		socks_fail("listen()", in, &ndst);
 
-#if defined(SO_OOBINLINE)
-	setsockopt(new, SOL_SOCKET, SO_OOBINLINE, &turnon, sizeof(turnon));
-#endif
 	socks_SendDst(in, &ndst);
 
 	len = sizeof(struct sockaddr_in);
 	if ((new = accept(out, (struct sockaddr *)&sin, &len)) < 0)
 		socks_fail("accept()", in, &ndst);
+#if defined(SO_OOBINLINE)
+	setsockopt(new, SOL_SOCKET, SO_OOBINLINE, &turnon, sizeof(turnon));
+#endif
 	close(out);
 
 	if (sin.sin_addr.s_addr == 0)
@@ -1230,7 +1231,11 @@ void reapChild()
 {
 	int status;
 
-	wait(&status);
+#if defined(NO_WAITPID)
+	while (wait3(&status, WNOHANG, NULL) > 0) ;
+#else
+	while (waitpid(-1, &status, WNOHANG) > 0) ;
+#endif
 	signal(SIGCHLD, reapChild);
 }
 /*
